@@ -4,13 +4,13 @@ import sys
 from collections import defaultdict
 from datetime import date, datetime
 from functools import wraps
-from typing import Any, Callable, DefaultDict, List, Optional, Sequence, Union
+from typing import Any, Callable, DefaultDict, List, Optional, Sequence, TextIO, Union
 
 import click
 import requests
 
 from . import __version__
-from .cve_api import CveApi
+from .cve_api import CveApi, CveRecordValidationError
 
 CVE_RE = re.compile(r"^CVE-[12]\d{3}-\d{4,}$")
 CONTEXT_SETTINGS = {
@@ -77,7 +77,7 @@ def print_table(lines: Sequence[Sequence[str]]) -> None:
 
 
 def print_json_data(data: Union[dict, list]) -> None:
-    click.echo(json.dumps(data, indent=4, sort_keys=True))
+    click.echo(json.dumps(data, indent=2, sort_keys=True))
 
 
 def print_user(user: dict) -> None:
@@ -97,8 +97,12 @@ def print_user(user: dict) -> None:
 
 
 def get_full_name(user_data: dict) -> Optional[str]:
-    name_data = user_data["name"]
-    return f"{name_data.get('first', '')} {name_data.get('last', '')}".strip() or None
+    # If no name values are defined on a user, the entire `name` object is not returned in the
+    # user data response; see https://github.com/CVEProject/cve-services/issues/901.
+    name = user_data.get("name", {})
+    if name:
+        return f"{name.get('first', '')} {name.get('last', '')}".strip() or None
+    return None
 
 
 def bool_to_text(value: Optional[bool]) -> str:
@@ -128,17 +132,25 @@ def handle_cve_api_error(func: Callable) -> Callable:
                     details = exc.response.json()
                 except ValueError:
                     details = exc.response.content
-
-        click.secho("ERROR: ", bold=True, nl=False)
-        click.echo(error)
-        if details:
-            click.secho("DETAILS: ", bold=True, nl=False)
-            click.echo(details)
+            print_error(error, details)
+        except CveRecordValidationError as exc:
+            click.secho("ERROR: ", bold=True, nl=False)
+            click.echo("CVE record is not valid against the v5 JSON schema:")
+            for error in exc.errors:
+                click.echo(f"  {error}")
         sys.exit(1)
 
     return wrapped
 
 
+def print_error(msg: str, details: Optional[str]) -> None:
+    click.secho("ERROR: ", bold=True, nl=False)
+    click.echo(msg)
+    if details:
+        click.secho("DETAILS: ", bold=True, nl=False)
+        click.echo(details)
+
+
 class Config:
     def __init__(
         self,
@@ -231,40 +243,69 @@ def cli(
     "-j",
     "--cve-json",
     "cve_json_str",
-    required=True,
     type=click.STRING,
     help="JSON body of CVE record to publish.",
 )
+@click.option(
+    "-f",
+    "--cve-json-file",
+    type=click.File(),
+    help="File containing JSON body of CVE record to publish.",
+)
 @click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
 @click.pass_context
 @handle_cve_api_error
-def publish(ctx: click.Context, cve_id: str, cve_json_str: str, print_raw: bool) -> None:
+def publish(
+    ctx: click.Context,
+    cve_id: str,
+    cve_json_str: Optional[str],
+    cve_json_file: Optional[TextIO],
+    print_raw: bool,
+) -> None:
     """Publish a CVE record for a reserved (or rejected) CVE ID.
 
     If the CVE is already published, this action will update its record. A published CVE can only be
     moved to the rejected state with an appropriate reject record (see `cve reject`). A published
     CVE cannot be moved back to the reserved state.
 
-    Example:
+    The CVE record can be specified as a string:
 
-    \b
-    cve publish CVE-2022-1234 -j '{"affected": [], "descriptions": [], "references": {}, ...}'
+      cve publish CVE-2022-1234 -j '{"affected": [], "descriptions": [], "references": {}, ...}'
 
-    For information on the required properties in a given CVE JSON record, see the
-    `cnaPublishedContainer` schema in:\n
+    Or passed in a file:
+
+      cve publish CVE-2022-1234 -f v5_record.json
+
+    For information on the required properties in a given CVE JSON record, see the schema in:\n
     https://github.com/CVEProject/cve-schema/blob/master/schema/v5.0/CVE_JSON_5.0_schema.json
+
+    Because the CVE Services API only expects the cnaPublishedContainer contents of the full record,
+    the record you pass to this command can specify just that data, and not the full record.
     """
+    if cve_json_file is not None and cve_json_str is not None:
+        raise click.BadParameter(
+            "cannot use both `-f/--cve-json-file` and `-j/--cve-json` to provide a CVE JSON record."
+        )
+
     try:
-        cve_json = json.loads(cve_json_str)
+        if cve_json_str is not None:
+            cve_json = json.loads(cve_json_str)
+        elif cve_json_file is not None:
+            cve_json = json.load(cve_json_file)
+        else:
+            raise click.BadParameter(
+                "must provide CVE JSON record using one of: "
+                "`-f/--cve-json-file` or `-j/--cve-json`."
+            )
     except json.JSONDecodeError as exc:
-        click.echo("CVE data was not valid JSON. Error was:\n")
-        click.secho(str(exc))
+        print_error(msg="CVE data is not valid JSON", details=str(exc))
         return
+
     if ctx.obj.interactive:
         click.echo("You are about to publish a CVE record for ", nl=False)
         click.secho(cve_id, bold=True, nl=False)
         click.echo(" using the following input:\n\n", nl=False)
-        click.secho(cve_json_str, bold=True, nl=False)
+        print_json_data(cve_json)
         if not click.confirm("\n\nDo you want to continue?"):
             click.echo("Exiting...")
             sys.exit(0)
@@ -296,10 +337,22 @@ def publish(ctx: click.Context, cve_id: str, cve_json_str: str, print_raw: bool)
     type=click.STRING,
     help="JSON body of CVE record to reject.",
 )
+@click.option(
+    "-f",
+    "--cve-json-file",
+    type=click.File(),
+    help="File containing JSON body of CVE record to reject.",
+)
 @click.option("--raw", "print_raw", default=False, is_flag=True, help="Print response JSON.")
 @click.pass_context
 @handle_cve_api_error
-def reject(ctx: click.Context, cve_id: str, cve_json_str: str, print_raw: bool) -> None:
+def reject(
+    ctx: click.Context,
+    cve_id: str,
+    cve_json_str: Optional[str],
+    cve_json_file: Optional[TextIO],
+    print_raw: bool,
+) -> None:
     """Reject a CVE record for a reserved or published CVE ID.
 
     If the CVE is already rejected, this action will update its record if one is supplied.
@@ -307,30 +360,42 @@ def reject(ctx: click.Context, cve_id: str, cve_json_str: str, print_raw: bool)
     A rejected CVE without a record can be moved to the reserved state. A published CVE can only
     be rejected with an accompanying record. Reserved CVEs can be rejected with or without a record.
 
-    Example:
+    The CVE reject record can be specified as a string:
 
-    \b
-    cve reject CVE-2022-1234 -j '{"rejectedReasons": [{"lang": "en", "value": "A reason."}]}'
+      cve reject CVE-2022-1234 -j '{"rejectedReasons": [{"lang": "en", "value": "A reason."}]}'
 
-    For information on the required properties in a given CVE JSON record, see the
-    `cnaRejectedContainer` schema in:\n
+    Or passed in a file:
+
+      cve reject CVE-2022-1234 -f v5_reject_record.json
+
+    For information on the required properties in a given CVE JSON record, see the schema in:\n
     https://github.com/CVEProject/cve-schema/blob/master/schema/v5.0/CVE_JSON_5.0_schema.json
+
+    Because the CVE Services API only expects the cnaRejectedContainer contents of the full record,
+    the record you pass to this command can specify just that data, and not the full record.
     """
-    cve_json = None
-    if cve_json_str:
-        try:
+    if cve_json_file is not None and cve_json_str is not None:
+        raise click.BadParameter(
+            "cannot use both `-f/--cve-json-file` and `-j/--cve-json` to provide a CVE JSON record."
+        )
+
+    try:
+        if cve_json_str is not None:
             cve_json = json.loads(cve_json_str)
-        except json.JSONDecodeError as exc:
-            click.echo("CVE data was not valid JSON. Error was:\n")
-            click.secho(str(exc))
-            return
+        elif cve_json_file is not None:
+            cve_json = json.load(cve_json_file)
+        else:
+            cve_json = None
+    except json.JSONDecodeError as exc:
+        print_error(msg="CVE data is not valid JSON", details=str(exc))
+        return
 
     if ctx.obj.interactive:
         click.echo("You are about to reject ", nl=False)
         click.secho(cve_id, bold=True, nl=False)
-        if cve_json:
+        if cve_json is not None:
             click.echo(" using the following input:\n\n", nl=False)
-            click.secho(cve_json_str)
+            print_json_data(cve_json)
         else:
             click.echo(" without providing a reject record.")
         if not click.confirm("\nDo you want to continue?"):
@@ -490,10 +555,10 @@ def show_cve(ctx: click.Context, show_record: bool, print_raw: bool, cve_id: str
                 raise exc
 
     if print_raw:
-        # Serialize the CVE ID data and CVE record into an array for easier parsing if we're
-        # showing the record as well. If not, just display the CVE ID data.
+        # Display CVE record data only if we're showing the record. Otherwise, show the CVE ID
+        # data only.
         if show_record:
-            print_json_data([cve_id_data, cve_record_data])
+            print_json_data(cve_record_data)
         else:
             print_json_data(cve_id_data)
     else:
